#pragma once

/*
* Simple Serial Protocol
*
* This product includes software developed by Nick Leeman.
* Simple Serial Protocol Project (https://git.aterve.com/Frozenverse/SimpleSerialProtocol).
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
*/

// Base Arduino Library
#include <Arduino.h>

#include "Packet.h"
#include "SerialConnection.h"

class SimpleSerialProtocol
{
    private:
        SerialConnection serialConnection;
    public:
        bool init(int deviceIndex, unsigned long baud);
        bool receivePackets(void (*callback)(Packet));
        bool sendPacket(char* ID, char* command, char* data);
};

bool SimpleSerialProtocol::init(int deviceIndex, unsigned long baud) {
    // Check if device index is in range
    if (deviceIndex < 0 || deviceIndex > 4) {
        return false;
    }

    // Init serial connection
    serialConnection.init(deviceIndex, baud);
    
    // Serial Connection Delay
    delay(5);

    return true;
};

bool SimpleSerialProtocol::receivePackets(void (*callback)(Packet)) {
    // Try to get buffer from serial device
    char* receivedBuffer = serialConnection.handle();

    // Check if received a complete buffer
    if (receivedBuffer != NULL) {
        int bufferSize = serialConnection.getBufferSize();

        // Buffer pointer
        char* strtokIndex;
        char id[bufferSize / 2];
        char command[bufferSize / 2];
        char data[bufferSize / 2];

        // Get ID
        strtokIndex = strtok(receivedBuffer, ":");
        strcpy(id, strtokIndex);

        // Get Command
        strtokIndex = strtok(NULL, ":");
        strcpy(command, strtokIndex);

        // Get Data
        strtokIndex = strtok(NULL, ":");
        strcpy(data, strtokIndex);

        // Pack received data into packet
        Packet packet;
        packet.id = id;
        packet.command = command;
        packet.data = data;

        // Call pointer callback function
        callback(packet);

        // Received complete buffer
        return true;
    }

    // Didn't receive complete buffer
    return false;
};

bool SimpleSerialProtocol::sendPacket(char* ID, char* command, char* data) {
    // Create new packet buffer
    char serialPacketBuffer[serialConnection.getBufferSize()];

    // Pack data in buffer
    sprintf(serialPacketBuffer, "%s%s:%s:%s%s", serialConnection.getPacketMarker(0), ID, command, data, serialConnection.getPacketMarker(1));

    // Send buffer to serial device
    serialConnection.send(serialPacketBuffer);
    return true;
};